#!/usr/bin/env python
# -*- coding: utf-8 -*-
"""
#==============================================================================
#     FileName: utils.py
#      License: MIT
#   LastChange: 9/29/2022 10:06
#    CreatedAt: 9/29/2022 10:06
#==============================================================================
"""
from typing import AnyStr
from base64 import b64encode, b64decode
from Crypto.Cipher import Salsa20

from django.conf import settings
from rest_framework.authentication import get_authorization_header

REST_SETTINGS = getattr(settings, "REST_FRAMEWORK", {})
VAULT_SETTINGS = REST_SETTINGS.get("REST_VAULT", {})
VAULT_KEY = VAULT_SETTINGS.get("SECRET_KEY", b"*Thirty-two byte (256 bits) key*")
VAULT_HTTP_CONTENT_TYPE = VAULT_SETTINGS.get("HTTP_CONTENT_TYPE", "application/vault")
VAULT_HTTP_ACCEPT = VAULT_SETTINGS.get("HTTP_ACCEPT", "application/vault")
VAULT_HTTP_FORMAT = VAULT_SETTINGS.get("HTTP_FORMAT", "vault")
VAULT_USE_TOKEN_AS_KEY = VAULT_SETTINGS.get("USE_TOKEN_AS_KEY", True)
# VAULT_ALGORITHM = VAULT_SETTINGS.get('ALGORITHM', 'salsa20')

if isinstance(VAULT_KEY, str):
    VAULT_KEY = VAULT_KEY.encode()


def get_key(request) -> bytes:
    if request is None:
        return VAULT_KEY

    auth = get_authorization_header(request).split()

    if not auth or len(auth) != 2:
        return VAULT_KEY

    try:
        token = auth[1].decode()
    except UnicodeError:
        return VAULT_KEY

    if len(token) < 32:
        return VAULT_KEY

    return token[31::-1].encode()


def encrypt(data: bytes, key: bytes = VAULT_KEY) -> bytes:
    """
    Encrypt data with algorithms Salsa20 & Base64.

    :param key: secret key
    :param data: plain text
    :return: encrypted data
    """
    if data is None:
        return bytes()

    cipher = Salsa20.new(key=key)
    encrypted_data = cipher.nonce + cipher.encrypt(data)
    return b64encode(encrypted_data)


def decrypt(data: AnyStr, key: bytes = VAULT_KEY) -> bytes:
    """
    Decrypt data with algorithms Salsa20 & Base64.

    :param key: secret key
    :param data: encrypted data
    :return: plain text
    """
    if data is None:
        return bytes()

    encrypted_data = b64decode(data)
    nonce = encrypted_data[:8]
    cipher_text = encrypted_data[8:]
    cipher = Salsa20.new(key=key, nonce=nonce)

    return cipher.decrypt(cipher_text)



#!/usr/bin/env python
# -*- coding: utf-8 -*-
"""
#==============================================================================
#     FileName: parsers.py
#      License: MIT
#   LastChange: 9/29/2022 09:58
#    CreatedAt: 9/29/2022 09:58
#==============================================================================
"""
import codecs

from django.conf import settings
from rest_framework.parsers import BaseParser
from rest_framework.exceptions import ParseError
from rest_framework.settings import api_settings
from rest_framework.utils import json

from .renderers import VaultRenderer
from .utils import decrypt, get_key, VAULT_HTTP_CONTENT_TYPE, VAULT_USE_TOKEN_AS_KEY


class VaultParser(BaseParser):
    """
    Parses Vault-serialized data.
    """

    media_type = VAULT_HTTP_CONTENT_TYPE
    renderer_class = VaultRenderer
    strict = api_settings.STRICT_JSON

    def parse(self, stream, media_type=None, parser_context=None):
        parser_context = parser_context or {}
        encoding = parser_context.get("encoding", settings.DEFAULT_CHARSET)

        try:
            decoded_stream = codecs.getreader(encoding)(stream)

            if VAULT_USE_TOKEN_AS_KEY:
                request = parser_context.get("request", None)
                key = get_key(request)
                data = decrypt(decoded_stream.read(), key)
            else:
                data = decrypt(decoded_stream.read())

            parse_constant = json.strict_constant if self.strict else None
            return json.loads(data, parse_constant=parse_constant)
        except ValueError as exc:
            raise ParseError("Data parse error - %s" % str(exc))



#!/usr/bin/env python
# -*- coding: utf-8 -*-
"""
#==============================================================================
#     FileName: renderers.py
#      License: MIT
#   LastChange: 9/29/2022 09:58
#    CreatedAt: 9/29/2022 09:58
#==============================================================================
"""
from rest_framework.renderers import BaseRenderer, JSONRenderer

from .utils import (
    encrypt,
    get_key,
    VAULT_HTTP_FORMAT,
    VAULT_HTTP_ACCEPT,
    VAULT_USE_TOKEN_AS_KEY,
)


class VaultRenderer(BaseRenderer):
    """
    Renderer which encrypts data.
    """

    media_type = VAULT_HTTP_ACCEPT
    format = VAULT_HTTP_FORMAT
    json_renderer = JSONRenderer()
    charset = "utf-8"

    def render(self, data, accepted_media_type=None, renderer_context=None):
        """
        Render `data` into encrypted data, returning a bytestring.
        """
        raw_json = self.json_renderer.render(
            data, accepted_media_type, renderer_context
        )
        renderer_context = renderer_context or {}

        if VAULT_USE_TOKEN_AS_KEY and renderer_context:
            request = renderer_context.get("request", None)
            key = get_key(request)
            return encrypt(raw_json, key)

        return encrypt(raw_json)



#!/usr/bin/env python
# -*- coding: utf-8 -*-
from __future__ import print_function

import io
import os
import re

from setuptools import setup


_pkg = "rest_framework_vault"


def get_version(package):
    """
    Return package version as listed in `__version__` in `init.py`.
    """
    init_py = open(os.path.join(package, "__init__.py")).read()
    return re.search("^__version__ = ['\"]([^'\"]+)['\"]", init_py, re.MULTILINE).group(
        1
    )


with io.open("README.md", encoding="utf-8") as readme:
    description = readme.read()

requirements = [
    "Django>=2.2.16",
    "djangorestframework>=3.10.3",
    "pycryptodome>=3.9.1",
]
VERSION = get_version(_pkg)

setup(
    name="django-rest-vault",
    version=VERSION,
    install_requires=requirements,
    packages=[_pkg],
    include_package_data=True,
    license="MIT License",
    description="Api data decryption and encryption support for Django REST Framework",
    long_description=description,
    long_description_content_type="text/markdown",
    test_suite="tests",
    url="https://github.com/wlemuel/django-rest-vault",
    author="Steve Lemuel",
    author_email="wlemuel@hotmail.com",
    keywords="drf django django-rest-framework",
    classifiers=[
        "Environment :: Web Environment",
        "Framework :: Django",
        "Framework :: Django :: 2.2",
        "Framework :: Django :: 3.0",
        "Framework :: Django :: 3.1",
        "Framework :: Django :: 3.2",
        "Framework :: Django :: 4.0",
        "Framework :: Django :: 4.1",
        "Intended Audience :: Developers",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
        "Programming Language :: Python",
        "Programming Language :: Python :: 3.6",
        "Programming Language :: Python :: 3.7",
        "Programming Language :: Python :: 3.8",
        "Programming Language :: Python :: 3.9",
        "Programming Language :: Python :: 3.10",
        "Programming Language :: Python :: 3.11",
        "Topic :: Internet :: WWW/HTTP",
        "Topic :: Internet :: WWW/HTTP :: Dynamic Content",
    ],
    zip_safe=False,
)
